/*
 * d_IIR.c
 *
 *  Created on: 2018328
 *      Author: Administrator
 */

#include "d_IIR.h"
#include <services\int\adi_int.h>
#include <stdlib.h>

extern volatile int acc_over;

extern float iir_input_left[];
extern float iir_input_right[];
extern float iir_output_left[];
extern float iir_output_right[];
extern float iir_coeffs_low_pass[];

// TCB Initialization
int IIR_TCB_RIGHT[13]={
                /*Coefficients and Dk values for a particular biquad BQD[k] should be
                  stored in internal memory in the order Ak0, Ak1, Bk1, Ak2, Bk2, Dk2,Dk1.*/
                0,                          // CP        = Chain pointer register
                7 * IIR_NUM_SECTIONS,       // CC/CL     = Coefficient buffer length register
                1,                          // CM        = Coefficient modifier register
                (int)iir_coeffs_low_pass,   // CI        = Coefficient index registerϵ洢ַ
                (int)iir_output_right,      // OB        = Output data base register
                IIR_AUDIO_BLOCK_SIZE,       // OL/OC     = Output buffer length register
                1,                          // OM        = Output modifier register
                (int)iir_output_right,      // OI        = Output data index register
                (int)iir_input_right,       // IB        = Input data base register
                IIR_AUDIO_BLOCK_SIZE,       // IL/IC     = Input buffer length register
                1,                          // IM        = Input buffer modifier register
                (int)iir_input_right,       // II        = Input data index register
                (IIR_NUM_SECTIONS - 1)|((IIR_AUDIO_BLOCK_SIZE-1)<<14)         // IIRCTL2   = Channel control register, no of biquads = 3, WINDOW SIZE=512
            };

int IIR_TCB_LEFT[13]={
                (int)IIR_TCB_RIGHT+12,       // CP        = Chain pointer register
                7 * IIR_NUM_SECTIONS,        // CC/CL     = Coefficient buffer length register
                1,                           // CM        = Coefficient modifier register
                (int)iir_coeffs_low_pass,    // CI        = Coefficient index register
                (int)iir_output_left,        // OB        = Output data base register
                IIR_AUDIO_BLOCK_SIZE,        // OL/OC     = Output buffer length register
                1,                           // OM        = Output modifier register
                (int)iir_output_left,        // OI        = Output data index register
                (int)iir_input_left,         // IB        = Input data base register
                IIR_AUDIO_BLOCK_SIZE,        // IL/IC     = Input buffer length register
                1,                           // IM        = Input buffer modifier register
                (int)iir_input_left,         // II        = Input data index register
                (IIR_NUM_SECTIONS - 1)|((IIR_AUDIO_BLOCK_SIZE-1)<<14)         // IIRCTL2   = Channel control register, no of biquads = 3, WINDOW SIZE=512
            };

void IIR_ACC_ISR();

/* Adding the Initialization Code for IIR Accelerator Now */
void init_IIR()
{
    int temp;

    //TCB
    IIR_TCB_RIGHT[0]=(int)IIR_TCB_LEFT+12;

    //Mapping the IIR DMA interrupt
    temp=*pPICR0;
    temp&=~(P0I0|P0I1|P0I2|P0I3|P0I4);
    temp|=P0I0|P0I1|P0I3|P0I4;
    *pPICR0=temp;
    //interrupt(SIG_P0,IIR_ACC_ISR);
    adi_int_InstallHandler(ADI_CID_P0I, (ADI_INT_HANDLER_PTR)IIR_ACC_ISR, NULL, true);

    //Selecting the IIR Accelerator
    temp=*pPMCTL1;
    temp&=~(BIT_17|BIT_18);
    temp|=IIRACCSEL;
    *pPMCTL1=temp;

    asm("nop;nop;nop;nop;");

    //Initializing the chain pointer register
    *pCPIIR=(int)IIR_TCB_LEFT+12-0x80000;

    //Now Enabling the IIR Accelerator
    *pIIRCTL1=IIR_EN|IIR_DMAEN|IIR_CH2|IIR_RND0;
}

void IIR_ACC_ISR()
{
    acc_over=1;
}




